@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -64,7 +64,8 @@
 /*!
     \fn QSize::QSize()
 
-    Constructs an invalid size.
+    Constructs a size with an invalid width and height (i.e., isValid()
+    returns false).
 
     \sa isValid()
 */
@@ -195,7 +196,7 @@ void QSize::scale(const QSize &s, Qt::AspectRatioMode mode)
         ht = s.ht;
     } else {
         bool useHeight;
-        int rw = s.ht * wd / ht;
+        int rw = qint32(qint64(s.ht) * qint64(wd) / qint64(ht));
 
         if (mode == Qt::KeepAspectRatio) {
             useHeight = (rw <= s.wd);
@@ -207,7 +208,7 @@ void QSize::scale(const QSize &s, Qt::AspectRatioMode mode)
             wd = rw;
             ht = s.ht;
         } else {
-            ht = s.wd * ht / wd;
+            ht = qint32(qint64(s.wd) * qint64(ht) / qint64(wd));
             wd = s.wd;
         }
     }
