@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -371,7 +371,7 @@ QPixmapData::~QPixmapData()
 
     A pixmap is automatically detached by Qt whenever its contents are
     about to change. This is done in almost all QPixmap member
-    functions that modify the pixmap (fill(), convertFromImage(),
+    functions that modify the pixmap (fill(), fromImage(),
     load(), etc.), and in QPainter::begin() on a pixmap.
 
     There are two exceptions in which detach() must be called
@@ -425,7 +425,7 @@ void QPixmap::fill(const QColor &fillColor)
 #ifndef QT_NO_XRENDER
         if (data->picture && data->d == 32) {
             ::Picture src  = X11->getSolidFill(data->xinfo.screen(), fillColor);
-            XRenderComposite(X11->display, PictOpSrc, src, 0, data->hd,
+            XRenderComposite(X11->display, PictOpSrc, src, 0, data->picture,
                              0, 0, width(), height(),
                              0, 0, width(), height());
         } else
@@ -475,11 +475,18 @@ QPixmap QPixmap::alphaChannel() const
     by converting the \a alphaChannel into 32 bit and using the
     intensity of the RGB pixel values.
 
+    The effect of this function is undefined when the pixmap is being
+    painted on.
+
     \sa alphaChannel(), {QPixmap#Pixmap Transformations}{Pixmap
     Transformations}
  */
 void QPixmap::setAlphaChannel(const QPixmap &alpha)
 {
+    if (paintingActive()) {
+        qWarning("QPixmap::setAlphaChannel: Should not set alpha channel while pixmap is being painted on");
+    }
+
     if (alpha.isNull())
         return;
 
@@ -536,16 +543,24 @@ QBitmap QPixmap::mask() const
 
     Setting a null mask resets the mask.
 
+    The effect of this function is undefined when the pixmap is being
+    painted on.
+
     \sa mask(), {QPixmap#Pixmap Transformations}{Pixmap
     Transformations}, QBitmap
 */
 void QPixmap::setMask(const QBitmap &newmask)
 {
+    if (paintingActive()) {
+        qWarning("QPixmap::setMask: Should not set mask while pixmap is being painted on");
+    }
+
     if (data == newmask.data)
         // trying to selfmask
         return;
 
     if (newmask.isNull()) { // clear mask
+        detach();
 #ifndef QT_NO_XRENDER
         if (data->picture && data->d == 32) {
             QPixmap pixmap;
@@ -676,7 +691,7 @@ int QPixmap::metric(PaintDeviceMetric m) const
     Note that for the moment, alpha masks on monochrome images are
     ignored.
 
-    \sa convertFromImage(), {QImage#Image Formats}{Image Formats}
+    \sa fromImage(), {QImage#Image Formats}{Image Formats}
 */
 
 QImage QPixmap::toImage() const
@@ -977,7 +992,7 @@ QPixmap QPixmap::fromImage(const QImage &img, Qt::ImageConversionFlags flags)
 {
     QPixmap pixmap;
     if (img.isNull()) {
-        qWarning("QPixmap::convertFromImage: Cannot convert a null image");
+        qWarning("QPixmap::fromImage: Cannot convert a null image");
         return pixmap;
     }
 
@@ -1632,7 +1647,7 @@ QPixmap QPixmap::fromImage(const QImage &img, Qt::ImageConversionFlags flags)
             free(newbits);
             newbits = (uchar *)newerbits;
         } else if (xi->bits_per_pixel != 8) {
-            qWarning("QPixmap::convertFromImage: Display not supported "
+            qWarning("QPixmap::fromImage: Display not supported "
                      "(bpp=%d)", xi->bits_per_pixel);
         }
         xi->data = (char *)newbits;
@@ -1684,7 +1699,7 @@ QPixmap QPixmap::fromImage(const QImage &img, Qt::ImageConversionFlags flags)
     copies everything to the bottom of the window.
 
     The window system identifier (\c WId) can be retrieved using the
-    QWidget::WId() function. The rationale for using a window
+    QWidget::winId() function. The rationale for using a window
     identifier and not a QWidget, is to enable grabbing of windows
     that are not part of the application, window system frames, and so
     on.
@@ -1703,7 +1718,7 @@ QPixmap QPixmap::fromImage(const QImage &img, Qt::ImageConversionFlags flags)
     \warning In general, grabbing an area outside the screen is not
     safe. This depends on the underlying window system.
 
-    \sa grabWidget()
+    \sa grabWidget(), {Screenshot Example}
 */
 
 QPixmap QPixmap::grabWindow(WId window, int x, int y, int w, int h)
