@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -20,7 +20,6 @@
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
-
 #include "qboxlayout.h"
 
 
@@ -154,10 +153,9 @@ void QBoxLayoutPrivate::setupGeom()
 
     int n = list.count();
     geomArray.clear();
-    geomArray.resize(n);
-    QVector<QLayoutStruct> &a = geomArray;
+    QVector<QLayoutStruct> a(n);
 
-    bool first = true;
+    bool first = true; // empty so far?
     for (int i = 0; i < n; i++) {
         QBoxLayoutItem *box = list.at(i);
         QSize max = box->item->maximumSize();
@@ -168,7 +166,6 @@ void QBoxLayoutPrivate::setupGeom()
         // space before non-empties, except the first:
         int space = (empty || first) ? 0 : q->spacing();
         bool ignore = empty && box->item->widget(); // ignore hidden widgets
-
         if (horz(dir)) {
             bool expand = exp & Qt::Horizontal || box->stretch > 0;
             horexp = horexp || expand;
@@ -176,8 +173,8 @@ void QBoxLayoutPrivate::setupGeom()
             minw += min.width() + space;
             hintw += hint.width() + space;
             if (!ignore)
-                qMaxExpCalc(maxh, verexp,
-                             max.height(), exp & Qt::Vertical);
+                qMaxExpCalc(maxh, verexp, first,
+                            max.height(), exp & Qt::Vertical, box->item->isEmpty());
             minh = qMax(minh, min.height());
             hinth = qMax(hinth, hint.height());
 
@@ -193,8 +190,8 @@ void QBoxLayoutPrivate::setupGeom()
             minh += min.height() + space;
             hinth += hint.height() + space;
             if (!ignore)
-                qMaxExpCalc(maxw, horexp,
-                             max.width(), exp & Qt::Horizontal);
+                qMaxExpCalc(maxw, horexp, first,
+                             max.width(), exp & Qt::Horizontal, box->item->isEmpty());
             minw = qMax(minw, min.width());
             hintw = qMax(hintw, hint.width());
 
@@ -207,8 +204,8 @@ void QBoxLayoutPrivate::setupGeom()
 
         a[i].empty = empty;
         hasHfw = hasHfw || box->item->hasHeightForWidth();
-        first = first && empty;
     }
+    geomArray = a;
 
     expanding = (Qt::Orientations)
                        ((horexp ? Qt::Horizontal : 0)
@@ -331,7 +328,7 @@ void QBoxLayoutPrivate::calcHfw(int w)
     \endlist
 
     The margin default is provided by the style. The default margin
-    Qt styles specify is 9 for widgets and 11 for top level windows.
+    most Qt styles specify is 9 for child widgets and 11 for windows.
     The spacing defaults to the same as the margin width for a
     top-level layout, or to the same as the parent layout.
 
@@ -1010,7 +1007,7 @@ QBoxLayout::Direction QBoxLayout::direction() const
 
     \image qhboxlayout-with-5-children.png Horizontal box layout with five child widgets
 
-    \sa QVBoxLayout, QGridLayout, QStackedLayout, {Layout Classes}
+    \sa QVBoxLayout, QGridLayout, QStackedLayout, {Layout Classes}, {Basic Layouts Example}
 */
 
 
@@ -1130,7 +1127,7 @@ QHBoxLayout::~QHBoxLayout()
 
     \image qvboxlayout-with-5-children.png Horizontal box layout with five child widgets
 
-    \sa QHBoxLayout, QGridLayout, QStackedLayout, {Layout Classes}
+    \sa QHBoxLayout, QGridLayout, QStackedLayout, {Layout Classes}, {Basic Layouts Example}
 */
 
 /*!
