@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -77,6 +77,28 @@
     Assigns the \a other table cell to this table cell.
 */
 
+/*!
+    \since 4.2
+
+    Sets the cell's character format to \a format. This can for example be used to change
+    the background color of the entire cell:
+
+    QTextTableCell cell = table->cellAt(2, 3);
+    QTextCharFormat format = cell.format();
+    format.setBackground(Qt::blue);
+    cell.setFormat(format);
+
+    \sa format()
+*/
+void QTextTableCell::setFormat(const QTextCharFormat &format)
+{
+    QTextCharFormat fmt = format;
+    fmt.clearProperty(QTextFormat::ObjectIndex);
+    QTextDocumentPrivate *p = table->docHandle();
+    QTextDocumentPrivate::FragmentIterator frag(&p->fragmentMap(), fragment);
+    p->setCharFormat(frag.position(), 1, fmt, QTextDocumentPrivate::SetFormatAndPreserveObjectIndices);
+}
+
 /*!
     Returns the cell's character format.
 */
@@ -98,11 +120,10 @@ int QTextTableCell::row() const
     if (tp->dirty)
         tp->update();
 
-    for (int i = 0; i < tp->nCols*tp->nRows; ++i) {
-        if (tp->grid[i] == fragment)
-            return i/tp->nCols;
-    }
-    return -1;
+    int idx = tp->findCellIndex(fragment);
+    if (idx == -1)
+        return idx;
+    return tp->cellIndices.at(idx) / tp->nCols;
 }
 
 /*!
@@ -116,11 +137,10 @@ int QTextTableCell::column() const
     if (tp->dirty)
         tp->update();
 
-    for (int i = 0; i < tp->nCols*tp->nRows; ++i) {
-        if (tp->grid[i] == fragment)
-            return i%tp->nCols;
-    }
-    return -1;
+    int idx = tp->findCellIndex(fragment);
+    if (idx == -1)
+        return idx;
+    return tp->cellIndices.at(idx) % tp->nCols;
 }
 
 /*!
@@ -272,16 +292,26 @@ QTextTable *QTextTablePrivate::createTable(QTextDocumentPrivate *pieceTable, int
     int charIdx = pieceTable->formatCollection()->indexForFormat(charFmt);
     int cellIdx = pieceTable->formatCollection()->indexForFormat(QTextBlockFormat());
 
-    for (int i = 0; i < rows*cols; ++i) {
-        pieceTable->insertBlock(QTextBeginningOfFrame, pos, cellIdx, charIdx);
+    QTextTablePrivate *d = table->d_func();
+    d->blockFragmentUpdates = true;
+
+    d->fragment_start = pieceTable->insertBlock(QTextBeginningOfFrame, pos, cellIdx, charIdx);
+    d->cells.append(d->fragment_start);
+    ++pos;
+
+    for (int i = 1; i < rows*cols; ++i) {
+        d->cells.append(pieceTable->insertBlock(QTextBeginningOfFrame, pos, cellIdx, charIdx));
 // 	    qDebug("      addCell at %d", pos);
         ++pos;
     }
 
-    pieceTable->insertBlock(QTextEndOfFrame, pos, cellIdx, charIdx);
+    d->fragment_end = pieceTable->insertBlock(QTextEndOfFrame, pos, cellIdx, charIdx);
 // 	qDebug("      addEOR at %d", pos);
     ++pos;
 
+    d->blockFragmentUpdates = false;
+    d->dirty = true;
+
     pieceTable->endEditBlock();
 
     return table;
@@ -295,12 +325,12 @@ struct QFragmentFindHelper
     const QTextDocumentPrivate::FragmentMap &fragmentMap;
 };
 
-static bool operator<(int fragment, const QFragmentFindHelper &helper)
+static inline bool operator<(int fragment, const QFragmentFindHelper &helper)
 {
     return helper.fragmentMap.position(fragment) < helper.pos;
 }
 
-static bool operator<(const QFragmentFindHelper &helper, int fragment)
+static inline bool operator<(const QFragmentFindHelper &helper, int fragment)
 {
     return helper.pos < helper.fragmentMap.position(fragment);
 }
@@ -318,6 +348,8 @@ int QTextTablePrivate::findCellIndex(int fragment) const
 void QTextTablePrivate::fragmentAdded(const QChar &type, uint fragment)
 {
     dirty = true;
+    if (blockFragmentUpdates)
+        return;
     if (type == QTextBeginningOfFrame) {
         Q_ASSERT(cells.indexOf(fragment) == -1);
         const uint pos = pieceTable->fragmentMap().position(fragment);
@@ -334,6 +366,8 @@ void QTextTablePrivate::fragmentAdded(const QChar &type, uint fragment)
 void QTextTablePrivate::fragmentRemoved(const QChar &type, uint fragment)
 {
     dirty = true;
+    if (blockFragmentUpdates)
+        return;
     if (type == QTextBeginningOfFrame) {
         Q_ASSERT(cells.indexOf(fragment) != -1);
         cells.removeAll(fragment);
@@ -359,6 +393,8 @@ void QTextTablePrivate::update() const
     QTextDocumentPrivate *p = pieceTable;
     QTextFormatCollection *c = p->formatCollection();
 
+    cellIndices.resize(cells.size());
+
     int cell = 0;
     for (int i = 0; i < cells.size(); ++i) {
         int fragment = cells.at(i);
@@ -372,12 +408,13 @@ void QTextTablePrivate::update() const
 
         int r = cell/nCols;
         int c = cell%nCols;
+        cellIndices[i] = cell;
 
-	if (r + rowspan > nRows) {
-	    grid = (int *)realloc(grid, sizeof(int)*(r + rowspan)*nCols);
-	    memset(grid + (nRows*nCols), 0, sizeof(int)*(r+rowspan-nRows)*nCols);
-	    nRows = r + rowspan;
-	}
+        if (r + rowspan > nRows) {
+            grid = (int *)realloc(grid, sizeof(int)*(r + rowspan)*nCols);
+            memset(grid + (nRows*nCols), 0, sizeof(int)*(r+rowspan-nRows)*nCols);
+            nRows = r + rowspan;
+        }
 
         Q_ASSERT(c + colspan <= nCols);
         for (int ii = 0; ii < rowspan; ++ii) {
@@ -813,33 +850,65 @@ void QTextTable::mergeCells(int row, int column, int numRows, int numCols)
 
     const int origCellPosition = cell.firstPosition() - 1;
 
+    const int cellFragment = d->grid[row * d->nCols + column];
+
     QVarLengthArray<int> cellMarkersToDelete((numCols - colSpan) * rowSpan
                                              + (numRows - rowSpan) * numCols);
 
     int idx = 0;
 
-    for (int r = row; r < row + rowSpan; ++r)
+    d->blockFragmentUpdates = true;
+
+    for (int r = row; r < row + rowSpan; ++r) {
+
+        const int fragment = d->grid[r * d->nCols + column + colSpan];
+        const uint pos = p->fragmentMap().position(fragment);
+        QFragmentFindHelper helper(pos, p->fragmentMap());
+        QList<int>::Iterator it = qBinaryFind(d->cells.begin(), d->cells.end(), helper);
+        Q_ASSERT(it != d->cells.end());
+        Q_ASSERT(*it == fragment);
+        d->cellIndices.remove(it - d->cells.begin(), numCols - colSpan);
+        d->cells.erase(it, it + numCols - colSpan);
+
         for (int c = column + colSpan; c < column + numCols; ++c) {
             const int cell = d->grid[r * d->nCols + c];
             QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
             cellMarkersToDelete[idx++] = it.position();
+            d->grid[r * d->nCols + c] = cellFragment;
         }
+    }
+
+    for (int r = row + rowSpan; r < row + numRows; ++r) {
+
+        const int fragment = d->grid[r * d->nCols + column];
+        const uint pos = p->fragmentMap().position(fragment);
+        QFragmentFindHelper helper(pos, p->fragmentMap());
+        QList<int>::Iterator it = qBinaryFind(d->cells.begin(), d->cells.end(), helper);
+        Q_ASSERT(it != d->cells.end());
+        Q_ASSERT(*it == fragment);
+        d->cellIndices.remove(it - d->cells.begin(), numCols);
+        d->cells.erase(it, it + numCols);
 
-    for (int r = row + rowSpan; r < row + numRows; ++r)
         for (int c = column; c < column + numCols; ++c) {
             const int cell = d->grid[r * d->nCols + c];
             QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
             cellMarkersToDelete[idx++] = it.position();
+            d->grid[r * d->nCols + c] = cellFragment;
         }
-
-    for (int i = 0; i < cellMarkersToDelete.size(); ++i) {
-        p->remove(cellMarkersToDelete[i] - i, 1);
     }
 
+    for (int i = 0; i < cellMarkersToDelete.size(); ++i)
+        p->remove(cellMarkersToDelete[i] - i, 1);
+
+    d->fragment_start = d->cells.first();
+
     fmt.setTableCellRowSpan(numRows);
     fmt.setTableCellColumnSpan(numCols);
     p->setCharFormat(origCellPosition, 1, fmt);
 
+    d->blockFragmentUpdates = false;
+    d->dirty = false;
+
     p->endEditBlock();
 }
 
@@ -912,7 +981,7 @@ void QTextTable::splitCell(int row, int column, int numRows, int numCols)
 
     int insertAdjustement = 0;
     for (int i = 0; i < numRows; ++i) {
-        for (int c = 0; c < colSpan - numCols; ++c) 
+        for (int c = 0; c < colSpan - numCols; ++c)
             p->insertBlock(QTextBeginningOfFrame, rowPositions[i] + insertAdjustement + c, blockIndex, fmtIndex);
         insertAdjustement += colSpan - numCols;
     }
