@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -21,8 +21,8 @@
 **
 ****************************************************************************/
 
-#ifndef QDOCKWIDGET_H
-#define QDOCKWIDGET_H
+#ifndef QDYNAMICDOCKWIDGET_H
+#define QDYNAMICDOCKWIDGET_H
 
 #include <QtGui/qwidget.h>
 
@@ -48,8 +48,8 @@ class Q_GUI_EXPORT QDockWidget : public QWidget
     Q_PROPERTY(QString windowTitle READ windowTitle WRITE setWindowTitle DESIGNABLE true)
 
 public:
-    explicit QDockWidget(const QString &title, QWidget *parent = 0, Qt::WFlags flags = 0);
-    explicit QDockWidget(QWidget *parent = 0, Qt::WFlags flags = 0);
+    explicit QDockWidget(const QString &title, QWidget *parent = 0, Qt::WindowFlags flags = 0);
+    explicit QDockWidget(QWidget *parent = 0, Qt::WindowFlags flags = 0);
     ~QDockWidget();
 
     QWidget *widget() const;
@@ -102,6 +102,7 @@ private:
     Q_PRIVATE_SLOT(d_func(), void _q_toggleTopLevel())
     friend class QDockWidgetLayout;
     friend class QDockWidgetItem;
+    friend class QMainWindowLayout;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QDockWidget::DockWidgetFeatures)
@@ -110,4 +111,4 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QDockWidget::DockWidgetFeatures)
 
 QT_END_HEADER
 
-#endif // QDOCKWIDGET_H
+#endif // QDYNAMICDOCKWIDGET_H
