@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -130,7 +130,7 @@ public:
     modal and \a f arguments are passed to the QDialog constructor.
 */
 
-Q3Wizard::Q3Wizard(QWidget *parent, const char *name, bool modal, Qt::WFlags f )
+Q3Wizard::Q3Wizard(QWidget *parent, const char *name, bool modal, Qt::WindowFlags f )
     : QDialog( parent, name, modal, f )
 {
     d = new Q3WizardPrivate();
