@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -88,9 +88,11 @@ void QDesignerIntegration::updateProperty(const QString &name, const QVariant &v
 
                 } else if (name == QLatin1String("geometry")) {
                     if (QWidget *container = containerWindow(formWindow)) {
-                        QRect r = containerWindow(formWindow)->geometry();
-                        r.setSize(value.toRect().size());
-                        container->setGeometry(r);
+                        SetFormPropertyCommand *cmd = new SetFormPropertyCommand(formWindow);
+                        cmd->init(object, name, value);
+                        cmd->setOldValue(container->geometry());
+                        formWindow->commandHistory()->push(cmd);
+
                         emit propertyChanged(formWindow, name, value);
                     }
 
