@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -30,7 +30,7 @@ class GammaView: public QWidget
 {
     Q_OBJECT
 public:
-    GammaView( QWidget *parent = 0, Qt::WFlags f = 0 )
+    GammaView( QWidget *parent = 0, Qt::WindowFlags f = 0 )
         : QWidget(parent,f) { }
 };
 
